import 'dart:async';
import 'dart:convert';
import 'dart:math';

import 'package:flutter/services.dart';
import 'package:n_bug_reporter/dio_client.dart';
import 'package:n_bug_reporter/model/create_task_template_body.dart';
import 'package:n_bug_reporter/model/clickup_task.dart';

import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart'; // In order to use RepaintBoundary, RenderRepaintBoundary
import 'dart:ui' as ui;
import 'dart:typed_data';

import 'package:package_info_plus/package_info_plus.dart';
import 'package:device_info_plus/device_info_plus.dart';

import 'model/clickup_custom_field_ids.dart';

// sources: https://codelabs.developers.google.com/codelabs/write-flutter-plugin
//  https://blog.logrocket.com/networking-flutter-using-dio/
class NBugReporter {
  static String _getRandString(int len) {
    var random = Random.secure();
    var values = List<int>.generate(len, (i) => random.nextInt(255));
    return base64UrlEncode(values);
  }

  static const MethodChannel _channel = MethodChannel('n_bug_reporter');

  static final DeviceInfoPlugin deviceInfoPlugin = DeviceInfoPlugin();

  // Package info has known issues: https://pub.dev/packages/package_info_plus
  static PackageInfo _packageInfo = PackageInfo(
    appName: 'Unknown',
    packageName: 'Unknown',
    version: 'Unknown',
    buildNumber: 'Unknown',
    buildSignature: 'Unknown',
  );
  static ClickUpCustomFieldIds _clickUpCustomFieldIds = ClickUpCustomFieldIds();
  static final DioClient _client = DioClient();
  static String _listId = '';
  static String _templateId = '';
  static GlobalKey? _repaintBoundaryKey;

  static Future<String?> get platformVersion async {
    final String? version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }

  static configure(
      {required String privateKey,
      required String listId,
      required String templateId,
      required ClickUpCustomFieldIds customFieldIds,
      required GlobalKey repaintBoundaryKey}) {
    _repaintBoundaryKey = repaintBoundaryKey;
    _listId = listId;
    _templateId = templateId;
    _clickUpCustomFieldIds = customFieldIds;
    _client.setAuthorizationHeader(privateKey);
  }

  static Future<ClickUpTask?> createTaskFromTemplate() async {
    CreateTaskTemplateBody body = CreateTaskTemplateBody(
      name: "New Bug Report with SS - #${_getRandString(5)}",
      tags: ["bug", "issue"],
      status: "Open",
      priority: 2,
      dueDateTime: false,
      startDate: DateTime.now().toUtc().millisecondsSinceEpoch,
      startDateTime: true,
      notifyAll: true,
      parent: null,
      linksTo: null,
      checkRequiredCustomFields: true,
    );
    _packageInfo = await PackageInfo.fromPlatform();
    var ssData = await getScreenshot(repaintBoundaryKey: _repaintBoundaryKey!);
    var createdTask = await _client.createTaskFromTemplate(
        listId: _listId, templateId: _templateId, body: body);
    String ssName = 'screenshot-${DateTime.now()}.png';
    await _client.createTaskAttachment(
        pngBytes: ssData, fileName: ssName, taskId: createdTask!.id);
    await _fillCustomFieldsForTask(taskId: createdTask!.id);
    return createdTask;
  }

  static _fillCustomFieldsForTask({required String taskId}) async {
    if (_clickUpCustomFieldIds.appBundleId != null) {
      await _client.setCustomFieldForTask(
          taskId: taskId,
          fieldId: _clickUpCustomFieldIds.appBundleId!,
          value: _packageInfo.packageName);
    }
    if (_clickUpCustomFieldIds.platform != null) {
      final _platformVersion = await platformVersion;
      await _client.setCustomFieldForTask(
          taskId: taskId,
          fieldId: _clickUpCustomFieldIds.platform!,
          value: _platformVersion ?? '');
    }
    if (_clickUpCustomFieldIds.deviceInfo != null) {
      final _deviceInfo = await deviceInfoPlugin.deviceInfo;
      final _deviceInfoStr = _deviceInfo
          .toMap()
          .entries
          .map((e) => e.value.toString())
          .toList()
          .join(' ');
      await _client.setCustomFieldForTask(
          taskId: taskId,
          fieldId: _clickUpCustomFieldIds.deviceInfo!,
          value: _deviceInfoStr);
    }
    if (_clickUpCustomFieldIds.activity != null) {
      await _client.setCustomFieldForTask(
          taskId: taskId,
          fieldId: _clickUpCustomFieldIds.activity!,
          value: 'ADD ACTIVITY NAME');
    }
  }

  // source: https://www.kindacode.com/article/how-to-programmatically-take-screenshots-in-flutter/
  static Future<Uint8List> getScreenshot(
      {required GlobalKey repaintBoundaryKey}) async {
    RenderRepaintBoundary boundary = repaintBoundaryKey.currentContext!
        .findRenderObject() as RenderRepaintBoundary;
    ui.Image image = await boundary.toImage();
    ByteData? byteData = await image.toByteData(format: ui.ImageByteFormat.png);
    if (byteData != null) {
      Uint8List pngBytes = byteData.buffer.asUint8List();
      return pngBytes;
    } else {
      throw Exception('N_BUG_REPORTER could not capture the screenshot!');
    }
  }

  static FloatingActionButton nBugReporterButton({Function? prereport,
  Function? postreport}) {
    return FloatingActionButton(
        onPressed: () async {
          if (prereport != null){
            await prereport();
          }
          await NBugReporter.createTaskFromTemplate();
          if (postreport != null){
            await postreport();
          }
        },
        backgroundColor: Colors.blueAccent,
        child: const Icon(Icons.keyboard_arrow_up));
  }
}
